part of 'vehicles_on_the_property.dart';

/// Widget que exibe o histórico de veículos na propriedade.
class VehicleHistoryScreen extends StatefulWidget {
  final VehicleModel model;

  VehicleHistoryScreen(this.model, {super.key});

  @override
  State<VehicleHistoryScreen> createState() => _VehicleHistoryScreenState();
}

class _VehicleHistoryScreenState extends State<VehicleHistoryScreen>
    with Remotable {
  final apiCall = PhpGroup.getVehiclesByProperty;

  @override
  void initState() {
    super.initState();
    _pagingController.addPageRequestListener(
      (dynamic pageKey) => _fetch(
        hasData: () async {
          final newItems = await apiCall.call(pageKey.toString());
          if (newItems.jsonBody == null) return (false, null);
          final List<dynamic> vehicles =
              (newItems.jsonBody['vehicles'] as List<dynamic>?) ?? [];
          _pagingController.nextPageKey = pageKey + 1;

          safeSetState(() {
            count = newItems.jsonBody['total_rows'] ?? 0;
          });
          return (vehicles.isNotEmpty, vehicles);
        },
        onNotHas: () {
          _showNoMoreDataSnackBar(context);
          setState(() {
            // _hasData = false;
            // _loading = false;
          });
        },
        onHas: (vehicles) {
          setState(() {
            // _loading = false;
          });
          _pagingController.appendLastPage(vehicles);
        },
        onError: (e, s) {
          DialogUtil.errorDefault(context);
          LogUtil.requestAPIFailed(
              "proccessRequest.php", "", "Consulta de Veículo", e, s);
          setState(() {
            // _hasData = false;
            // _loading = false;
          });
        },
      ),
    );
    _pagingController.addStatusListener(_showError);
  }

  @override
  void dispose() {
    _pagingController.dispose();
    super.dispose();
  }

  Future<void> _showError(PagingStatus status) async {
    if (status == PagingStatus.subsequentPageError) {
      final message = FFLocalizations.of(context).getVariableText(
        enText: 'Something went wrong while fetching a new page.',
        ptText: 'Algo deu errado ao buscar uma nova página.',
      );
      final retry = FFLocalizations.of(context).getVariableText(
        enText: 'Retry',
        ptText: 'Recarregar',
      );

      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          content: Text(message),
          action: SnackBarAction(
            label: retry,
            onPressed: () => _pagingController.retryLastFailedRequest(),
          ),
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        _buildHeader(context),
        _buildBody(),
      ],
    );
  }

  Widget _buildHeader(BuildContext context) {
    final bodyFontSize = LimitedFontSizeUtil.getBodyFontSize(context);
    return SizedBox(
      height: 30,
      child: Center(
        child: Text(
          (widget.model.amountRegister == '0' ||
                  widget.model.amountRegister == null)
              ? ''
              : "${FFLocalizations.of(context).getVariableText(ptText: "Quantidade de Veículos: ", enText: "Amount of Vehicles: ")}${widget.model.amountRegister}/$count",
          textAlign: TextAlign.right,
          style: TextStyle(
            fontFamily: 'Nunito',
            fontSize: bodyFontSize,
          ),
        ),
      ),
    );
  }

  Expanded _buildBody() {
    final noDataFound = FFLocalizations.of(context).getVariableText(
      ptText: "Nenhum veículo encontrado!",
      enText: "No vehicle found",
    );
    return Expanded(
      child: RefreshIndicator(
        onRefresh: () async => _pagingController.refresh(),
        child: PagedListView<int, dynamic>(
          pagingController: _pagingController,
          builderDelegate: PagedChildBuilderDelegate<dynamic>(
            animateTransitions: true,
            itemBuilder: (context, item, index) =>
                _generateItems(context, index, item),
            // noMoreItemsIndicatorBuilder: ,
            newPageProgressIndicatorBuilder: (context) =>
                _buildLoadingIndicator(context),
            firstPageProgressIndicatorBuilder: (context) =>
                _buildLoadingIndicator(context),
            noItemsFoundIndicatorBuilder: (context) =>
                _buildNoDataFound(context, noDataFound),
            firstPageErrorIndicatorBuilder: (context) => const Placeholder(),
            newPageErrorIndicatorBuilder: (context) => const Placeholder(),
          ),
        ),
      ),
    );
  }

  Widget _generateItems(BuildContext context, int index, dynamic item) {
    log('item: $item');
    final bool? isOwner = item['isOwnerVehicle'];
    final IconData iconData =
        isOwner == true ? Symbols.garage : Symbols.directions_car;

    final FreCardIcon? cardIcon = isOwner != null
        ? FreCardIcon(
            height: 50,
            width: 100,
            icon: Icon(iconData, size: 80, opticalSize: 10),
          )
        : null;

    final String? tag = item['tag'];
    final bool containTag = tag.isNotNullAndEmpty;
    final Map<String, String> labelsHashMap =
        _generateLabelsHashMap(context, item, tag, containTag);

    final List<Map<String, Color>> statusHashMapList =
        _generateStatusHashMapList(item);

    Future<void> onTapCardItemAction() async {
      await _handleCardItemTap(context, cardIcon, item);
    }

    final statusLinkedHashMap = statusHashMapList
        .map((map) => LinkedHashMap<String, Color>.from(map))
        .toList();
    final length = statusLinkedHashMap.expand((e) => [e.length]);
    final double itemWidthFactor = length == 1 ? 0.25 : 0.50;

    return CardItemTemplateComponentWidget(
      icon: cardIcon,
      labelsHashMap: labelsHashMap,
      statusHashMap: statusHashMapList,
      onTapCardItemAction: onTapCardItemAction,
      itemWidthFactor: itemWidthFactor,
    );
  }

  Map<String, String> _generateLabelsHashMap(
    BuildContext context,
    Map<String, dynamic> item,
    String? tag,
    bool containTag,
  ) {
    final localization = FFLocalizations.of(context);
    return {
      '${localization.getVariableText(ptText: "Placa", enText: "License Plate")}:':
          item['licensePlate'] ?? '',
      '${localization.getVariableText(ptText: "Modelo", enText: "Model")}:':
          item['model'] ?? '',
      '${localization.getVariableText(ptText: "Proprietário", enText: "Owner")}:':
          item['personName'] ?? '',
      if (containTag)
        '${localization.getVariableText(ptText: "Tag", enText: "Tag")}:':
            tag ?? '',
    };
  }

  List<Map<String, Color>> _generateStatusHashMapList(
      Map<String, dynamic> item) {
    final statusHashMap = widget.model.generateStatusColorMap(item, false);
    return statusHashMap != null ? [statusHashMap] : [];
  }

  Future<void> _handleCardItemTap(
    BuildContext context,
    FreCardIcon? cardIcon,
    Map<String, dynamic> item,
  ) async {
    try {
      final dialogContent = widget.model.buildVehicleDetails(
        icon: cardIcon,
        item: item,
        context: context,
        model: widget.model,
      );

      await showDialog(
        useSafeArea: true,
        context: context,
        builder: (context) => Dialog(
          alignment: Alignment.center,
          child: dialogContent,
        ),
      ).whenComplete(() {
        safeSetState(() {
          // _pagingController.refresh();
        });
      });
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed(
          "proccessRequest.php", "", "Consulta de Veículos", e, s);
      safeSetState(() {
        // _hasData = false;
        // _loading = false;
      });
    }
  }
}

mixin Remotable on State<VehicleHistoryScreen> {
  final PagingController<int, dynamic> _pagingController =
      PagingController<int, dynamic>(firstPageKey: 1);

  double offset = 0.0;
  // bool _hasData = true;
  // bool _loading = true;
  int count = 0;

  Future<void> _fetch({
    required Future<(bool, dynamic)> Function() hasData,
    required void Function(dynamic) onHas,
    required void Function() onNotHas,
    required void Function(Object, StackTrace) onError,
  }) async {
    // if (!_hasData || !_loading) return;
    try {
      final (bool, dynamic) data = await hasData();
      if (data.$1)
        onHas(data.$2);
      else
        onNotHas();
    } catch (e, s) {
      onError(e, s);
    }
  }

  void _showNoMoreDataSnackBar(BuildContext context) {
    final message = FFLocalizations.of(context).getVariableText(
      ptText: "Não há mais dados.",
      enText: "No more data.",
    );

    showSnackbar(context, message, true);
  }

  Widget _buildNoDataFound(BuildContext context, String title) {
    final headerFontSize = LimitedFontSizeUtil.getHeaderFontSize(context);
    // final bodyFontSize = LimitedFontSizeUtil.getBodyFontSize(context);
    return Expanded(
      child: Center(
        child: Text(
          title,
          style: TextStyle(
            fontFamily: 'Nunito',
            fontSize: headerFontSize,
          ),
        ),
      ),
    );
  }

  Widget _buildLoadingIndicator(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(vertical: 15),
      child: Center(
        child: CircularProgressIndicator(
          valueColor: AlwaysStoppedAnimation<Color>(
            FlutterFlowTheme.of(context).primary,
          ),
        ),
      ),
    );
  }
}
